DROP DATABASE IF EXISTS Strutture;

CREATE DATABASE IF NOT EXISTS Strutture DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

USE Strutture;

CREATE TABLE Utenti (
	Username VARCHAR(50) PRIMARY KEY,
	Password VARCHAR(15) NOT NULL,
	Nome CHAR(50) NOT NULL,
	Cognome CHAR(50)NOT NULL,
	dataNascita DATE NOT NULL,
	tipoUtente CHAR(14) NOT NULL
) ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Mostre (
	Codice INT(11) AUTO_INCREMENT PRIMARY KEY,
	Nome VARCHAR(50) NOT NULL,
	Descrizione VARCHAR (1000) NOT NULL
) ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Spettacoli (
	Codice INT(11) AUTO_INCREMENT PRIMARY KEY,
	Titolo VARCHAR(50) NOT NULL,
	Produttore VARCHAR(50) NOT NULL,
	Regista VARCHAR(50) NOT NULL,
	Durata INT(3) NOT NULL,
	Genere VARCHAR(50) NOT NULL
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Film (
	Codice INT(11) AUTO_INCREMENT PRIMARY KEY,
	Nome VARCHAR(50) NOT NULL,
	annoProduzione INT(11) NOT NULL,
	Regista VARCHAR(50) NOT NULL,
	Durata INT(3) NOT NULL,
	Genere VARCHAR(50) NOT NULL,
	Locandina BLOB NOT NULL
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Recensioni (
	Titolo VARCHAR(50) NOT NULL,
	Timestamp TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
	Valutazione INT(11) NOT NULL,
	Testo VARCHAR(1000) NOT NULL, 
	codiceFilm INT(11) NULL,
	codiceMostra INT(11) NULL,
	codiceSpettacolo INT(11) NULL,
	Username VARCHAR(50),
	FOREIGN KEY (Username) REFERENCES Utenti(Username) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (codiceFilm) REFERENCES Film(Codice) ON DELETE SET NULL ON UPDATE CASCADE,
	FOREIGN KEY (codiceMostra) REFERENCES Mostre(Codice) ON DELETE SET NULL ON UPDATE CASCADE,
	FOREIGN KEY (codiceSpettacolo) REFERENCES Spettacoli(Codice) ON DELETE SET NULL ON UPDATE CASCADE,
	PRIMARY KEY(Username, Timestamp),
	CHECK (Valutazione>=0 AND Valutazione<=5)
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Segnalazioni (
	Codice INT(11) AUTO_INCREMENT PRIMARY KEY,
	Descrizione VARCHAR(50) NOT NULL,
	Data DATE NOT NULL,
	Orario TIME NOT NULL,
	Latitudine VARCHAR(30) NOT NULL,
	Longitudine VARCHAR(30) NOT NULL,
	maxPartecipanti INT(11) NOT NULL,
	Organizzatore VARCHAR(50) NOT NULL,
	FOREIGN KEY(Organizzatore) REFERENCES Utenti(Username) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Prenotazioni (
	codiceSegnalazione INT(11),
	Utente VARCHAR(50),
	PRIMARY KEY(codiceSegnalazione, Utente),
	FOREIGN KEY(codiceSegnalazione) REFERENCES Segnalazioni(Codice) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY(Utente) REFERENCES Utenti(Username) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Musei (
	Nome VARCHAR(50) PRIMARY KEY,
	Indirizzo VARCHAR(200) NOT NULL,
	Telefono VARCHAR(11) NOT NULL,
	Latitudine VARCHAR(30) NOT NULL,
	Longitudine VARCHAR(30) NOT NULL,
	Foto BLOB NOT NULL,
	Website VARCHAR(50) NOT NULL,
	Descrizione VARCHAR(500) NOT NULL,
	oraApertura TIME NOT NULL,
	oraChiusura TIME NOT NULL,
	numSale INT(3) NOT NULL
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Teatri (
	Nome VARCHAR(50) PRIMARY KEY,
	Indirizzo VARCHAR(200) NOT NULL,
	Telefono VARCHAR(11) NOT NULL,
	Latitudine VARCHAR(30) NOT NULL,
	Longitudine VARCHAR(30) NOT NULL,
	Foto BLOB NOT NULL,
	Website VARCHAR(50) NOT NULL,
	Capienza INT(4) NOT NULL
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Cinema (
	Nome VARCHAR(50) PRIMARY KEY,
	Indirizzo VARCHAR(200) NOT NULL,
	Telefono VARCHAR(11) NOT NULL,
	Latitudine VARCHAR(30) NOT NULL,
	Longitudine VARCHAR(30) NOT NULL,
	Foto BLOB NOT NULL,
	Website VARCHAR(50) NOT NULL
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Mostre_Musei (
	codiceMostra INT(11),
	nomeMuseo VARCHAR(50),
	dataInizio DATE,
	dataFine DATE NOT NULL,
	Prezzo FLOAT(6,2) NOT NULL,
	PRIMARY KEY (codiceMostra, nomeMuseo, dataInizio),
	FOREIGN KEY (codiceMostra) REFERENCES Mostre(Codice) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (nomeMuseo) REFERENCES Musei(Nome) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Proiezioni_Teatrali (
	codiceSpettacolo INT(11),
	nomeTeatro VARCHAR(50),
	Orario TIME NOT NULL,
	Prezzo FLOAT(6,2) NOT NULL,
	Data DATE,
	PRIMARY KEY (codiceSpettacolo, nomeTeatro, Data),
	FOREIGN KEY (codiceSpettacolo) REFERENCES Spettacoli(Codice) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (nomeTeatro) REFERENCES Teatri(Nome) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Lista_Sale (
	Codice INT(11) AUTO_INCREMENT PRIMARY KEY,
	nomeCinema VARCHAR(50) NOT NULL,
	Capienza INT(11) NOT NULL,
	Nome VARCHAR(50) NOT NULL,
	FOREIGN KEY (nomeCinema) REFERENCES Cinema(Nome) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Proiezioni_Cinema (
	codiceFilm INT(11),
	codiceSala INT(11),
	Orario TIME,
	Prezzo FLOAT(6,2) NOT NULL,
	Data DATE,
	PRIMARY KEY (codiceFilm, codiceSala, Orario, Data),
	FOREIGN KEY (codiceFilm) REFERENCES Film(Codice) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (codiceSala) REFERENCES Lista_Sale(Codice) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;

CREATE TABLE Follower (
	Codice INT(11) AUTO_INCREMENT PRIMARY KEY,
	Utente VARCHAR(50) NOT NULL,
	nomeMuseo VARCHAR(50) NULL,
	nomeTeatro VARCHAR(50) NULL,
	nomeCinema VARCHAR(50) NULL,
	FOREIGN KEY (Utente) REFERENCES Utenti(Username) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (nomeMuseo) REFERENCES Musei(Nome) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (nomeTeatro) REFERENCES Teatri(Nome) ON DELETE CASCADE ON UPDATE CASCADE,
	FOREIGN KEY (nomeCinema) REFERENCES Cinema(Nome) ON DELETE CASCADE ON UPDATE CASCADE
)ENGINE=INNODB DEFAULT CHARACTER SET utf8 COLLATE utf8_general_ci;